/**********************************
 * KASRSDK 封装帮助类
 * 依赖：
 * https://open.kedacom.com/api/kasr/web-demo/kasrsdk/kasrsdk.js
 * https://open.kedacom.com/api/kasr/web-demo/common/crypto-js.js
 * https://github.com/taisel/XAudioJS/blob/master/resampler.js
 **********************************/
"use strict";

const TARGET_SAMPLERATE = 16000;
const KASR_URL = 'wss://asr.kedacom.com/kasr';
const APP_ID = 'krtctest';
const APP_SECRET = 'krtctest';

// 正式环境,应该从服务器获取token.
function getKasrToken(app_id = "krtctest", app_secret = "krtctest") {
    const version = '1';
    const user_id = '';
    const expire = 7200;

    let salt = Math.floor(Math.random() * 100000000);
    let expire_ts = new Date().getTime() + expire;

    const signature = CryptoJS.enc.Base64.stringify(CryptoJS.HmacSHA256(app_id + user_id + salt + expire_ts, app_secret));

    return version + ":" + app_id + ":" + salt + ":" + expire_ts + ":" + signature
}

function KasrHelper({ onRecogResult }) {
    this.kasrUrl = KASR_URL;
    this.appToken = getKasrToken(APP_ID, APP_SECRET);
    this.audioContext = null;
    this.audioSource = null;
    this.audioProcessor = null;
    this.kasrClient = null;
    this.onRecogResult = onRecogResult || function() {};

    this.initialize();
}

// 初始化处理
KasrHelper.prototype.initialize = function () {
    let that = this;
    let tmpResult = '';
    let finResult = '';
    //初始化库
    KASRSDK.init({ isEnableLog: true });
    that.kasrClient = KASRSDK.createAsrInstance();
    that.kasrClient.init({
        onStart: function (result, sessionId) {
            if (result != KASRSDK.CODE.ASR_ERROR_NONE) {
                if (KASRSDK.CODE.ASR_ERROR_NET == result)
                    alert('服务器连接失败，服务器地址可能不对!');
                else if (KASRSDK.CODE.ASR_ERROR_SESSION_DUP == result)
                    alert('授权token已经存在连接!');
                else if (KASRSDK.CODE.ASR_ERROR_TOKEN == result)
                    alert('授权验证失败，可能token不正确!');
                else
                    alert('服务器异常');

                    that.stopRecorder();
            }
        },
        onError: function (result, content) {
            alert('服务器发生错误!');
            that.stopRecorder();
        },
        onRecogResult: function ({ text = '', fin = 0 }) {
            if (fin == 0) {
                tmpResult = text;
            } else {
                finResult += text + '\r\n';
                tmpResult = ''
            }
            that.onRecogResult && that.onRecogResult(finResult + tmpResult, { text, fin });
        }
    });

    if (that.kasrClient.start({ serveraddr: that.kasrUrl, token: that.appToken }) != KASRSDK.CODE.ASR_ERROR_NONE) {
        alert('服务器连接失败');
        that.stopRecorder();
    }
}

// 开始音频识别处理
KasrHelper.prototype.startRecorder = function( { audioTrack, targetSampleRate=TARGET_SAMPLERATE} ){
    let that = this;
    if (!audioTrack) {
        console.log("startRecorder error: %o, %o", audioTrack);
        return;
    }

    let audioStream = new MediaStream();
    audioStream.addTrack(audioTrack);

    that.audioContext = new AudioContext();
    that.audioSource = that.audioContext.createMediaStreamSource(audioStream); 
    that.audioProcessor = that.audioContext.createScriptProcessor(4096,1,1)
    that.audioProcessor.onaudioprocess = function(e){
        // 获取输入的音频
        let inputBuffer = e.inputBuffer;
        // console.log("inputBuffer: %o", inputBuffer);

        // 将重新采样的音频转换为PCM
        let pcmData = that.inputBufferToPcmData(inputBuffer, targetSampleRate);

        // 对PCM数据进行处理
        that.kasrClient.feed({
            data: pcmData
        });
    }
    that.audioSource.connect(that.audioProcessor)
    that.audioProcessor.connect(that.audioContext.destination)
}

// 停止音频识别处理
KasrHelper.prototype.stopRecorder = function () {   
    let that = this;
    if (that.audioProcessor != null) {
        that.audioSource.disconnect(that.audioProcessor)
        that.audioProcessor.disconnect(that.audioContext.destination)   
        that.audioContext = null;  

        that.kasrClient.stop();
        KASRSDK.destoryAsrInstance(that.kasrClient);
        KASRSDK.release();
        that.audioProcessor = null;
    }
}

// 转换麦克风输入为PCM
KasrHelper.prototype.inputBufferToPcmData = function (inputBuffer, targetSampleRate) {
    // 对获取到的音频数据做重新采样处理
    let channelData = inputBuffer.getChannelData(0);
    let resampler = new Resampler(inputBuffer.sampleRate, targetSampleRate, 1, channelData);
    let outputOffset = resampler.resampler(channelData.length);

    // 设置输出的audioBuffer
    let resampleData = resampler.outputBuffer.slice(0, outputOffset);

    // float32Array转换为Int16Array
    var length = resampleData.length;
    var i16array = new Int16Array(length)
    for (var i = 0; i < length; i++) {
        i16array[i] = this.convert(resampleData[i])
    }

    return (new Uint8Array(i16array.buffer)).buffer;
}

// 转换float32为int16
KasrHelper.prototype.convert = function (n) {
    var v = n < 0 ? n * 32768 : n * 32767;       // convert in range [-32768, 32767]
    return Math.max(-32768, Math.min(32768, v)); // clamp
}
